<?php
namespace Controller;

use Util\Messages;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Util\Utilities;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;


class TaskType extends \Generated\Controller\TaskType {

    /** @var  \Service\Task $taskService */
    protected $taskService;

    public function __construct(Manager $dataSourceManager, Utilities $utilities, \Service\Task $taskService){
        $this->utilities = $utilities;
        $this->taskService = $taskService;
        parent::__construct($dataSourceManager);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION", "=", $this->utilities->getCurrentOrganizationId());
        parent::find($request,$response);
        $rows = $response->getDataSets()[0]->getRows();
        foreach ($rows as &$row) {
            $row['ISSUE_CLASSIFICATION'] = Utilities::setSelectMultiInQuery($row['ISSUE_CLASSIFICATION']) ;
            $row['TASK_CLASSIFICATION'] = Utilities::setSelectMultiInQuery($row['TASK_CLASSIFICATION']) ;
            $row['ID'] = intval($row['ID']);
        }
        $response->addDataSet(new DataSet($this->dataSourceName, $rows));
    }

    public function validateAddTaskClassification(DTO\Request\Filter $request, DTO\Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        $taskType = Utilities::getValueInFilterDataSourceByName($filters,'TASK_TYPE');
        $dataSet = $this->taskService->validateAddTaskClassification($taskType);
        DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
    }

    public function validateAddProjectType(DTO\Request\Filter $request, DTO\Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        $taskType = Utilities::getValueInFilterDataSourceByName($filters,'TASK_TYPE');
        $dataSet = $this->taskService->validateAddProjectType($taskType);
        DevOpsUtilities::setDataSetDefault($request, $response, $dataSet);
    }

	public function deleteTaskIssue(DTO\Request\Row $request, DTO\Response $response) {
        try{
            $row = $request->getRow();
            $id = $row['ID'];

            $this->taskService->removeTaskType($id);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
        } catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function save(DTO\Request $request, DTO\Response $response, $showMessage = true) {
        try{
            $taskType = $request->getDataSet()->getRows();
            $row = $taskType[0];
            $id = $row['ID'];
            $control = $row['CONTROL'];
            $label = $row['LABEL'];
            $allowCloseIssue = $row['ALLOW_CLOSE_ISSUE'];
            $orderToView = filter_var($row['ORDER_TO_VIEW'],FILTER_SANITIZE_NUMBER_INT);
            $dayNumbers = $row['DAY_NUMBERS'];
            $taskClassification = $row['TASK_CLASSIFICATION'];
            $allowPlayTask = $row['ALLOW_PLAY_TASK'];
            $issueClassification = $row['ISSUE_CLASSIFICATION'];
            $comment = $row['DESCRIPTION'];
            $targetStatus = empty($row['TARGET_STATUS'])?null:$row['TARGET_STATUS'];
            $dataSet = $this->taskService->updateTaskType($id, $control, $label, $allowCloseIssue,$orderToView,$dayNumbers,
                $taskClassification, $allowPlayTask, $issueClassification,$comment,$targetStatus);

            $response->addDataSet(new DataSet('taskType', $dataSet));
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
        } catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}