<?php 
namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities AS DevOpsUtilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Devops\Entities as EntitiesDevops;
use Zeedhi\DevOps\Model\Devops\Entities as DevOpsEntities;
use Util\Utilities;
use Util\Messages;

class ProductVersion
{
    /**
     * @var EntityManager
     */
    private $entityManager;
    
    public function __construct(EntityManager $entityManager)
    {
         $this->entityManager = $entityManager;
         $this->environment = InstanceProvider::getEnvironment();
    }
    
    public function findProductVersion($conditions, $page, $pageSize)
    {
        $projectId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID', true);
        $filterString = DevOpsUtilities::createQueryFiltered($conditions);
        $product = InstanceProvider::getRepository()->getProduct()->findOneBy(array('project' => $projectId));
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return InstanceProvider::getRepository()->getProductVersion()->findProductVersionConcatenated($product->getId(), $organizationId, $limit, $offset, $filterString);
    } 

    public function findProductVersionLtsWithoutBuild($conditions, $page, $pageSize)
    {
        $projectId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID', true);
        $resourceId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'RESOURCE_ID', true);

        $filterNotLTS = 0;
        if($resourceId && $projectId){
            $filterNotLTS = $this->getFilterVersionNotLTS($resourceId, $projectId);
        }

        /** @var Entities\Project $project */
        $project = InstanceProvider::getRepository()->getProject()->find($projectId);
        $projectTypeProduct = InstanceProvider::getService()->getProject()->getProductProject($project);
        $product = InstanceProvider::getRepository()->getProduct()->findOneBy(array('project' => $projectTypeProduct->getId()));

        $filterString = DevOpsUtilities::createQueryFiltered($conditions);
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();

        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return InstanceProvider::getRepository()->getProductVersion()
            ->findProductVersionLtsConcatenatedWithoutBuild($product->getId(), $organizationId, $limit, $offset, $filterString, $filterNotLTS);
    }

    public function findLtsCurrentProductVersionFromVersion($conditions)
    {
        $version = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'VERSION', true);
        if(preg_match('/^(\d+\.)?(\d+\.)?(\*|\d+)$/', $version)){
            $revision  = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'REVISION', true);
            $projectId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID', true);
            $isNext    = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'IS_NEXT', true);
            $product   = InstanceProvider::getRepository()->getProduct()->findOneBy(array('project' => $projectId));
            $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
            $productVersionsLts = InstanceProvider::getRepository()->getProductVersion()->findLtsCurrentProductVersionFromVersion($product->getId(), $organizationId, $version, $revision, $isNext);
            return $productVersionsLts;
        } else {
            return array();
        }
    }

    /**
     * @param $conditions
     * @return array
     */
    public function findProductVersionLtsByProject($conditions)
    {
        $projectId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PROJECT_ID', true);
        $resourceId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'RESOURCE_ID', true);
        $parentProjectId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PARENT_PROJECT_ID', true);
        /** @var Entities\Project $project */
        $project = $product = InstanceProvider::getRepository()->getProject()->find($projectId);
        $filterNotLTS = 0;
        if($resourceId && $parentProjectId){
            $filterNotLTS = $this->getFilterVersionNotLTS($resourceId, $parentProjectId);
        }

        if(!empty($project)){
            $projectTypeProduct = InstanceProvider::getService()->getProject()->getProductProject($project);
            if(!empty($projectTypeProduct)){
                $product = InstanceProvider::getRepository()->getProduct()->findOneBy(array('project' => $projectTypeProduct->getId()));
                $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
                return InstanceProvider::getRepository()->getProductVersion()
                    ->findProductVersionLtsConcatenatedWithoutBuild($product->getId(), $organizationId, 500, 0, '', $filterNotLTS);
            } else {
                return array();
            }
        } else {
            return array();
        }
    }

    public function findCurrentBuildFromVersion($conditions)
    {
        /** @var EntitiesDevops\ProductVersion $productVersion */
        $productVersionId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_VERSION_ID', true);
        $productVersion   = InstanceProvider::getRepository()->getProductVersion()->find($productVersionId);
        $organizationId   = InstanceProvider::getEnvironment()->getOrganizationId();
        $productId        = $productVersion->getProduct()->getId();
        $version          = $productVersion->getVersion();
        $revision         = $productVersion->getRevision();
        return InstanceProvider::getRepository()->getProductVersion()->findCurrentBuildFromVersion($productId, $organizationId, $version, $revision);
    }

    public function findNextLTSFromVersion($conditions)
    {
        /** @var EntitiesDevops\ProductVersion $productVersion */
        $productVersionId = DevOpsUtilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_VERSION_ID', true);
        if(empty($productVersionId)) {
            return array();
        }
        $productVersion   = InstanceProvider::getRepository()->getProductVersion()->find($productVersionId);
        $organizationId   = InstanceProvider::getEnvironment()->getOrganizationId();
        $productId        = $productVersion->getProduct()->getId();
        $version          = $productVersion->getVersion();
        $revision         = $productVersion->getRevision();
        return InstanceProvider::getRepository()->getProductVersion()->findLtsCurrentProductVersionFromVersion($productId, $organizationId, $version, $revision, $productVersionId);
    }

    /**
     * @param $projectId
     * @return Entities\Project|null
     */
    private function getProductProjectByProjectId($projectId)
    {
        /** @var Entities\Project $project */
        $project = InstanceProvider::getRepository()->getProject()->find($projectId);
        if($project){
            return InstanceProvider::getService()->getProject()->getProductProject($project);
        }
        return null;
    }

    
    public function removeLTSFromVersions($rows) {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
       
        try {
            
            $productVersionIds = Utilities::getArrayFromArrayAssociative($rows, 'ID');
            $productVersionIdsFilter = Utilities::getStringForArray($productVersionIds);
            $isNotHomologated = InstanceProvider::getRepository()->getProductVersion()->versionWithRelationshipWithHomologatedSubRelease($productVersionIdsFilter);
            
            if(empty($isNotHomologated)){
                foreach ($rows as $row) {
                    /** @var DevOpsEntities\ProductVersion $productVersion */
                    $productVersion = InstanceProvider::getRepository()->getProductVersion()->find($row['ID']);
                    $productVersion->setIsLts(DevOpsEntities\ProductVersion::IS_LTS_NO);
                    $entityManager->persist($productVersion);
                }
    
                $entityManager->flush();
                $connection->commit();
            } else {
                $msg = Messages::NO_PERMISSION_REMOVE_LTS_VERSIONS;
                foreach($isNotHomologated as $version) {
                    $msg .= '<li>' . $version['VERSION'] . '</li>';
                }
                throw new \Exception($msg);
            }
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public static function getProductEntityFromProject(Entities\Project $project = null)
    {
        $productEntity = null;
        if ($project) {
            if ($project->getProductVersion()) {
                $productEntity = $project->getProductVersion()->getProduct();
            } else {
                $productEntity = self::getProductEntityFromProject($project->getParent());
            }
        }
        return $productEntity;
    }
    
    public function findLTSProductVersionToRemove($projectId){
        $projectRepository = InstanceProvider::getRepository()->getProject();
        $ltsInfo = [];
        $version = InstanceProvider::getRepository()->getProductVersion()->findProdcutVersionByProject($projectId);
        $projectEntity = InstanceProvider::getRepository()->getProject()->find($projectId);
        $productProject = $this->getProductEntityFromProject($projectEntity);

        if($version){
            $version[0]['VERSION'] = intval($version[0]['VERSION']);
            $version[0]['REVISION'] = intval($version[0]['REVISION']);
            $lts = InstanceProvider::getRepository()->getProductVersion()->findLTSProductVersionToRemoveByVersion($productProject->getId(), $version[0]);
            $ltsInfo['first_value'] = array_shift($lts);
            $ltsInfo['lts'] = $lts;
        }
        
        return $ltsInfo;
    }

    private function getFilterVersionNotLTS($resourceId, $projectId)
    {
        $nrOrg = InstanceProvider::getRepository()->getResource()->findOrganizationByResourceId($resourceId);
        $productProject = $this->getProductProjectByProjectId($projectId);
        if($productProject && !empty($nrOrg)){
            $product = InstanceProvider::getRepository()->getProduct()->findOneBy(array('project' => $productProject->getId()));
            $omProduto = InstanceProvider::getRepository()->getOmProduto()->findOneBy(array('nrexternalProductId' => $product->getId()));
            $nrOrgId = $nrOrg['NRORG'];
            if ($omProduto) {
                $nrprodutoId = $omProduto->getId();
                $filterNotLTS = "SELECT OV.PRODUCT_VERSION_ID 
                                   FROM OM_PRODUTO_ORG OP
                                   JOIN OM_PRODUTO_ORG_VERSAO OV ON OV.NRPRODUTO_ORG_ID = OP.ID
                                  WHERE OP.NRORG_ID = $nrOrgId
                                    AND OP.NRPRODUTO_ID = $nrprodutoId";
                return $filterNotLTS;
            }
            return 0;
        }
        return 0;
    }

}